import os
import openai
# from langchain.llms import AzureOpenAI
from langchain.agents import create_pandas_dataframe_agent
import pyspark.pandas as ps
import json
from langchain.chat_models import AzureChatOpenAI

class Standardization:
    def __init__(self,columns,df,api_key,api_base,api_version,deployment_model,model_name):
        self.columns=columns
        self.df=df
        self.api_key=api_key
        self.api_base=api_base
        self.api_version=api_version
        self.deployment_model=deployment_model
        self.model_name=model_name

    
    def initiate_langchainFN(self):
            os.environ["OPENAI_API_TYPE"] = "azure"

            os.environ["OPENAI_API_KEY"] = self.api_type

            os.environ["OPENAI_API_BASE"] = self.api_base

            os.environ["OPENAI_API_VERSION"] = self.api_version


            llm = AzureChatOpenAI(

                    openai_api_type="azure",

                    deployment_name=self.deployment_model,

                    model_name=self.model_name,

                    temperature=0)

            #agent = create_pandas_dataframe_agent(llm, df, verbose=False, max_iterations=10)
            return llm
    def format_issue_detection(self):
        prompt= """ For the column "{0}", extract the following information:
                represents: Describe What does the column "{0}" represents?
                
                exists:String variable. The value should be "True"/"False".\
                Check if all the values are in uniform formats or are there any standardization issues exists in the column "{0}"?
                Keeping in mind what the column represents , think logically about the possible standardization issues\
                it can have from the below listed ones.\
                standardization issues can be any of the below:
                - issue1: different languages
                - issue2: different formats
                - issue3: different capitalization 
                - issue4: different units of measurement
                - issue5: different representations
                - issue6: abbreviated and expanded form
                - issue7: different metrics
                - issue8: symbols like @ represented in different ways
                used for different values within the column. And use your common sense to detect other kind of standardization issues
                that exists in the column that are not listed above. Set the value to 'True' if any of the issues exists,\
                'False' if no issues exists or unknown. Don't resolve the formatting issues. Your task is to detect the issues.

                issue: If "exists:" is true, give brief description of what the issue is from your analysis. format is given below:\
                ["issue1": description, "issue2": description, "issue3": description, etc.]
                description should be formatted as Triple-Quoted Strings.
                if "exists' is false, the value should be [issue:'None']. 
                
                sample: If "exists" is true,\
                By analyzing the "issues" you have found above, identify the different formats present in the data and\
                output exactly one value from the data corresponding to each different formats.\
                Output a maximum of 7 different formats that are most frequent and 7 examples, one example per format.
                Remember, within a issue, different formats or representations may occur.\
                Make sure to use escape characters wherever necessary to not cause any errors
    
                Output structure is given below:\
                [format1:example1, format2:example2, format3:example3, ..,formatN:exampleN]
                where format is the different formats and example is the example for each formats. 
                You need to present the format in an intuitive manner/general form
                instead of simply giving the data itself as format. You can use the letter x if there is no other way to represent the format.
                
                if "exists' is false, set this to "None".


                percentage: out of the total data, accurately provide how much percentage of data is present in each of these formats you found above in a descending manner.
                [format1:percentage of data, format2:percentage of data, format3:percentage of data, ..,formatN:percentage of data]
                Format the Final answer as dictionary with the following keys:
                represents
                exist 
                issue
                sample
                percentage

                continue response generation until the json is complete
                """
        final=[]
        columns=self.columns
        for column in columns:
            if self.df[column].dtype == object:
                self.df[column] = self.df[column].str.strip()
            llm=self.initiate_langchainFN()
            agent = create_pandas_dataframe_agent(llm, self.df[[column]], verbose=True, max_iterations=15, early_stopping_method="generate")
            prompt_ = prompt.format(column)
            output = agent.run(prompt_)
            print(output)
            data_dict = json.loads(output)
            data_dict["column_name"] = column
            final.append(data_dict)
        columns_order=["column_name","represents", "exists", "issue", "sample", "percentage"]
        final_df = ps.DataFrame(final, columns=columns_order)
        
        # data = [Row(**dict(zip(columns_order, final)))]


        return final_df


import openai
import os
from langchain.llms import AzureOpenAI
import pyspark.pandas as ps
import json
from langchain.agents import create_pandas_dataframe_agent
from langchain.agents import create_spark_dataframe_agent

class BusinessContext:
    def __init__(self,api_key,api_base,api_version,deployment_model,model_name):
                
        
        """ Initialize the class
            Params:
            agent: Langchain agent which can be use to get the result from ai model using prompt as input.
        """

        self.api_key=api_key
        self.api_base=api_base
        self.api_version=api_version
        self.deployment_model=deployment_model
        self.model_name=model_name


    def initiate_langchainFN(self):
            os.environ["OPENAI_API_TYPE"] = "azure"

            os.environ["OPENAI_API_KEY"] = self.api_key

            os.environ["OPENAI_API_BASE"] = self.api_base

            os.environ["OPENAI_API_VERSION"] = self.api_version


            llm = AzureOpenAI(

                    openai_api_type="azure",

                    deployment_name=self.deployment_model,

                    model_name=self.model_name,

                    temperature=0)

            #agent = create_pandas_dataframe_agent(llm, df, verbose=False, max_iterations=10)
            return llm

    def business_contextFN(self,df):
        llm=self.initiate_langchainFN()
        #agent = create_pandas_dataframe_agent(llm,df, verbose=False, max_iterations=10)
        agent=create_spark_dataframe_agent(llm,df,verbose=False, max_iterations=10)
        prompt = """Understand the given dataset and generate the business context of the data.
            The Business context of the data will include the name, description, the domain of data and the entities or attributes associated with the data.
            The output should be in json structure with above mentioned points as keys. Please give a little detailed explanation in values and explain the attributes. 
            Return the final answer in json format.
            """
        llm=self.initiate_langchainFN()
        agent = create_pandas_dataframe_agent(llm, df, verbose=False, max_iterations=10)
        business_context=agent.run(prompt)

        ##-test--
        model_output = business_context[business_context.find('{'):]

        model_output = model_output.replace("'", "\"")

        #model_output = eval(model_output)
        model_output = json.loads(model_output.replace("'", "\""))

        output_df = ps.DataFrame(model_output)

        output_df = output_df.transpose()
        #business_context = ps.DataFrame([business_context], columns=['Business_Context'])
        #return business_context
        return output_df
    
    def dq_rulesFN(self,df):
        prompt="""
        Understand the given dataset and generate a set of data quality checks which can be applied on the dataset. The information about the checks is as following-
        Null check- If the value of the column should not be Null for a record. Example of null check- for a policy data, Null check can be applied for the policy id and name but cannot be applied for duration.
        Unique check- If the value of the column should be Unique for a record. For Example for a employee data, unique check can be applied for -employeeid  but not for name and salary.
        Pattern check- If some regex pattern is supposed to be checked for the record. Example of pattern check- a regex for Checking if first alphabet of a Name column value is capital, regex to check if a phone number column has all digits, similarly for gender and date columns.
        The result should be a json with below structure-
        {
        "Null check": ["Column1", "Column2", ...],
        "Unique check": ["Column3", "Column4", ...],
        "Pattern check":{"Column1":"pattern1","Column2":"pattern2",...}
        }
        """
        llm=self.initiate_langchainFN()
        agent = create_pandas_dataframe_agent(llm, df, verbose=False, max_iterations=10)
        dq_rules=agent.run(prompt)
        df = ps.DataFrame([dq_rules],columns=['Data_Quality_Rules'])
        return dq_rules